unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ExtCtrls;

type
  TMainForm = class(TForm)
    BaseGroup0: TGroupBox;
    Panel3: TPanel;
    PulseOutLabel3: TLabel;
    PulseOutTrack3: TTrackBar;
    Panel4: TPanel;
    PulseOutLabel4: TLabel;
    PulseOutTrack4: TTrackBar;
    Panel5: TPanel;
    PulseOutLabel5: TLabel;
    PulseOutTrack5: TTrackBar;
    BaseGroup1: TGroupBox;
    Panel6: TPanel;
    PulseOutLabel6: TLabel;
    PulseOutTrack6: TTrackBar;
    Panel7: TPanel;
    PulseOutLabel7: TLabel;
    PulseOutTrack7: TTrackBar;
    Panel8: TPanel;
    PulseOutLabel8: TLabel;
    PulseOutTrack8: TTrackBar;
    BaseGroup2: TGroupBox;
    Panel9: TPanel;
    PulseOutLabel9: TLabel;
    PulseOutTrack9: TTrackBar;
    Panel10: TPanel;
    PulseOutLabel10: TLabel;
    PulseOutTrack10: TTrackBar;
    Panel11: TPanel;
    PulseOutLabel11: TLabel;
    PulseOutTrack11: TTrackBar;
    procedure FormCreate(Sender: TObject);
    procedure PulseOutTrackChange(Sender: TObject);
  protected
    DeviceIndex: LongWord;
    TicksPerCycle: array[0..2] of Word;
    BaseGroup: array[0..2] of TGroupBox;
    PulseOutLabel: array[3..11] of TLabel;
    PulseOutTrack: array[3..11] of TTrackBar;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  AIOUSB,
  DetectUnit;

procedure TMainForm.FormCreate(Sender: TObject);
const
  RootClock = 10000000;

var
  DetectForm: TDetectForm;
  Status: LongWord;
  PID: LongWord;
  iBase, iCounter: Integer;
  BaseFreq: array[0..2] of Double;

begin
  DeviceIndex := diOnly;

  BaseGroup[0] := BaseGroup0;
  BaseGroup[1] := BaseGroup1;
  BaseGroup[2] := BaseGroup2;
  PulseOutLabel[3 ] := PulseOutLabel3 ;
  PulseOutLabel[4 ] := PulseOutLabel4 ;
  PulseOutLabel[5 ] := PulseOutLabel5 ;
  PulseOutLabel[6 ] := PulseOutLabel6 ;
  PulseOutLabel[7 ] := PulseOutLabel7 ;
  PulseOutLabel[8 ] := PulseOutLabel8 ;
  PulseOutLabel[9 ] := PulseOutLabel9 ;
  PulseOutLabel[10] := PulseOutLabel10;
  PulseOutLabel[11] := PulseOutLabel11;
  PulseOutTrack[3 ] := PulseOutTrack3 ;
  PulseOutTrack[4 ] := PulseOutTrack4 ;
  PulseOutTrack[5 ] := PulseOutTrack5 ;
  PulseOutTrack[6 ] := PulseOutTrack6 ;
  PulseOutTrack[7 ] := PulseOutTrack7 ;
  PulseOutTrack[8 ] := PulseOutTrack8 ;
  PulseOutTrack[9 ] := PulseOutTrack9 ;
  PulseOutTrack[10] := PulseOutTrack10;
  PulseOutTrack[11] := PulseOutTrack11;

  Status := QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);
  if (Status <> ERROR_SUCCESS) or (PID <> $8020) then begin
    DetectForm := TDetectForm.Create(self);
    if DetectForm.ShowModal = mrOk then begin
      DeviceIndex := DetectForm.DeviceIndex;
      DetectForm.Free;
    end
    else begin
      DetectForm.Free;
      Application.Terminate;
      Exit;
    end;
  end;

  //Set the base frequencies. Min base frequency is 152.588 Hz.
  BaseFreq[0] := 1000;
  BaseFreq[1] := 500;
  BaseFreq[2] := 250;

  for iBase := 0 to 2 do begin
    //Set up base counter.
    BaseGroup[iBase].Caption := 'Group ' + IntToStr(iBase) + ' Base Freq: ' + FloatToStr(BaseFreq[iBase]);
    TicksPerCycle[iBase] := Round(RootClock / BaseFreq[iBase]); //Ticks of the 10MHz root clock per PWM cycle.
    CTR_8254ModeLoad(DeviceIndex, 0, iBase, 2, TicksPerCycle[iBase]);

    //Set up the counters based on it.
    for iCounter := (iBase+1)*3 to (iBase+1)*3 + 2 do begin
      PulseOutTrack[iCounter].Max := TicksPerCycle[iBase];
      PulseOutTrack[iCounter].Frequency := TicksPerCycle[iBase] div 100;
      PulseOutTrack[iCounter].PageSize := TicksPerCycle[iBase] div 10;
      PulseOutTrack[iCounter].OnChange := PulseOutTrackChange;
      PulseOutTrackChange(PulseOutTrack[iCounter]);
    end;
  end;

  //Shut down the unused counters.
  for iCounter := 12 to 14 do
    CTR_8254Mode(DeviceIndex, 0, iCounter, 0)
  ;
end;

procedure TMainForm.PulseOutTrackChange(Sender: TObject);
var
  iCounter, iBase: Integer;
  HighCt, LowCt: Integer;

begin
  iCounter := (Sender as TControl).Tag;
  if iCounter < Low(PulseOutTrack) then Exit;
  if iCounter > High(PulseOutTrack) then Exit;

  iBase := (iCounter div 3) - 1;

  HighCt := PulseOutTrack[iCounter].Position;
  LowCt := TicksPerCycle[iBase] - HighCt;

  if LowCt = 0 then
    CTR_8254Mode(DeviceIndex, 0, iCounter, 1)
  else
    CTR_8254ModeLoad(DeviceIndex, 0, iCounter, 1, LowCt)
  ;

  PulseOutLabel[iCounter].Caption := 'Counter ' + IntToStr(iCounter) + ': ' + FloatToStr(100 * HighCt / TicksPerCycle[iBase]) + '%';
end;

end.
